﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using SingularSys.Jep;
using SingularSys.Jep.ConfigurableParser;
using SingularSys.Jep.Reals;
using SingularSys.Jep.Standard;
using SingularSys.Jep.Types;
using System.Collections;

namespace JepTests
{
    [TestFixture]
    public class ExpressionEvalTest
    {
        
	    /** Switch for printing expressions while running tests */
	    static protected Boolean printExpressions = true;
    	
	    /** Current line position */
	    int lineCount;
    	
	    /**
	     * Creates a new ExpressionEvalTest instance
	     */
	    public ExpressionEvalTest() {

	    }
    	
	    /**
	     * The main method checks the arguments and creates an instance
	     * and runs test(s) contained in the argument string
	     */
	    public static void main(String[] args) {
		    String testString;
    		
		    // get filename from argument, or use default
		    if (args!=null && args.Length>0) {
			    testString = args[0];
		    } else {
			    testString = "JEPTestExpressions.txt";
		    }
    		
		    // Create an instance of this class and analyse the file
		    ExpressionEvalTest jt = new ExpressionEvalTest();
		    try {
		        jt.testStandard();
		    } catch (Exception e) {
		        Console.Out.WriteLine(e.ToString());
		    }
	    }

	    [Test]
        // TODO - throws exception - add comment
	    public void testStandard() {
		    JepInstance jep;
		    Console.Out.WriteLine("--------------------------------------------------------------");
            jep = new JepInstance();

		    jep.ImplicitMul = true;
		    jep.VarTab.Remove("true");
		    jep.VarTab.Remove("false");
		    jep.AddVariable("true", true);
		    jep.AddVariable("false", false);
		    lineCount = 0;

		    testJepInstance(jep, JepTests.Properties.Resources.JEPTestExpressions);
	    }

	    [Test]
	    // TODO - throws exception - add comment
	    public void testConfParser()  {
            JepInstance jep;
		    Console.Out.WriteLine("--------------------------------------------------------------");
            jep = new JepInstance();
            jep.Parser = new StandardConfigurableParser();

		    jep.ImplicitMul = true;
		    jep.VarTab.Remove("true");
		    jep.VarTab.Remove("false");
		    jep.AddVariable("true", true);
		    jep.AddVariable("false", false);
		    lineCount = 0;

            testJepInstance(jep, JepTests.Properties.Resources.JEPTestExpressions);
	    }

	    [Test]
	    // TODO - throws exception - add comment
	    public void testPostfixEvaluator() {
            JepInstance jep;
		    Console.Out.WriteLine("--------------------------------------------------------------");
            jep = new JepInstance();

            // TODO - was: jep.SetComponent(new PostfixEvaluator());

		    jep.ImplicitMul = true;
		    jep.VarTab.Remove("true");
		    jep.VarTab.Remove("false");
		    jep.AddVariable("true", true);
		    jep.AddVariable("false", false);
		    lineCount = 0;

            testJepInstance(jep, JepTests.Properties.Resources.JEPTestExpressions);
	    }

	    [Test]
	    // TODO - throws exception - add comment
	    public void testReals() {
            JepInstance jep;
    		
		    Console.Out.WriteLine("--------------------------------------------------------------");
            jep = new JepInstance();

            jep.Evaluator = new RealEvaluator();

		    jep.ImplicitMul = true;
		    jep.VarTab.Remove("true");
		    jep.VarTab.Remove("false");
		    jep.AddVariable("true", true);
		    jep.AddVariable("false", false);
		    lineCount = 0;

		    testJepInstance(jep, JepTests.Properties.Resources.JEPTestExpressionsReals);
	    }
    	

	    /**
	     * Loads the file specified in fileName. Evaluates the expressions listed
	     * in it and compares the expressions with the results.
	     */
	    public void testJepInstance(JepInstance jep, String testString) {
		    StringReader reader;
		    Object v1, v2;
		    String expression1, expression2;
		    Boolean hasError = false;

		    // Load the input string
		    try {
			    reader = new StringReader(testString);
		    } catch (Exception e) {
                println("Error loading testString: " + e.Message);
                Assert.Fail("Error loading testString: " + e.Message);
			    return;
		    }
    		
		    // reset the line count
		    lineCount = 0;
    		
		    // cycle through the expressions in pairs of two
		    println("Evaluating and comparing expressions...");
		    while (true) {
			    v1 = null;
			    v2 = null;
    			
			    // get values of a pair of two lines
			    try {
				    expression1 = getNextLine(reader);
				    expression2 = getNextLine(reader);
				    if (expression1 != null && expression2 != null) {
					    if (printExpressions) 
						    Console.Out.WriteLine("Checking \"" + expression1 + "\" == \"" + expression2 + "\"?");
					    v1 = parseNextLine(jep, expression1);
					    v2 = parseNextLine(jep, expression2);
				    }
			    } catch (Exception e) {
				    println("Exception occured: "+e.Message);
				    Console.Out.WriteLine(e.ToString());
				    hasError = true;
				    //break;
				    continue;
			    }

			    // expression1 or expression2 is null when end of file is reached
			    if (expression1 == null || expression2 == null) {
				    println("Reached end of file.");
				    break;
			    }
    	
			    // compare the results
			    if (!equal(v1, v2)) {
				    hasError = true;
				    println("Line: " + lineCount + ": ");
				    println("\"" + expression1 + "\" (" + v1.ToString() + ") != \"" 
						        + expression2 + "\" (" + v2.ToString()+")");
			    }
		    }
    		
		    // Closing remarks
		    print("\n" + lineCount + " lines processed. ");
		    if (hasError) {
			    print("Errors were found.\n\n");
		    } else {
			    print("No errors were found.\n\n");
		    }
    		
		    // Fail if errors are found
		    Assert.IsTrue(!hasError, "Errors were found.");
	    }
    	
	    /**
	     * Reads the next line from the Reader into a  String.
	     * @throws Exception when IOException occurs, parsing fails, or when
	     *         evaluation fails
	     */
        // TODO - throws exception - add comment
	    private String getNextLine(StringReader reader) {
		    String line;
    		
		    // cycle till a valid line is found
		    do {
			    line = reader.ReadLine(); // returns null on end of file
			    if (line == null) return null;
			    lineCount++;
		    } while (line.Length == 0 || line.Trim()[0] == '#');
    		
		    return line;
	    }
    	
	    /**
	     * Parses a single line from the reader, and returns the
	     * evaluation of that line.
	     * @return the value of the evaluated line. Returns null when the end of the file
	     *         is reached.
	     * @throws Exception when parsing fails, or when
	     *         evaluation fails
	     */
	    // TODO - throws exception - add comment
	    private Object parseNextLine(JepInstance jep, String line) {
		    Object value;
		    String errorStr;
		    // parse the expression
		    try {
			    jep.Parse(line);
		    } catch (ParseException e) {
			    // an error occur while parsing
			    errorStr = e.Message;
			    throw new Exception("Error while parsing line " + lineCount + ": " + errorStr);
		    }
    		
		    // evaluate the expression
		    try {
			    value = jep.Evaluate();
			    if (value == null) throw new EvaluationException("Evaluated as null");
		    } catch (EvaluationException e) {
			    errorStr = e.Message;
                errorStr += e.StackTrace;
			    throw new Exception(">>>Error while evaluating line " + lineCount + ": " + errorStr);
		    }

		    return value;
	    }

	    /**
	     * Compares o1 and o2. Copied from Comparative.java.
	     * @return true if o1 and o2 are equal. false otherwise.
	     */
	    private Boolean equal(Object param1, Object param2)
	    {
		    double tolerance = 1e-15;
		    if ((param1 is Complex) && (param2 is Complex)) {
			    return ((Complex)param1).IsEqual((Complex)param2, tolerance);
		    }
		    if ((param1 is Complex) && (param2 is JepDouble)) {
                return ((Complex)param1).IsEqual(new Complex((JepDouble)param2), tolerance);
		    }
		    if ((param2 is Complex) && (param1 is JepDouble)) {
                return ((Complex)param2).IsEqual(new Complex((JepDouble)param1), tolerance);
		    }
            if ((param1 is JepDouble) && (param2 is JepDouble))
            {
                return Math.Abs(((JepDouble)param1).DoubleValue - ((JepDouble)param2).DoubleValue)
					    < tolerance;
		    }
            if ((param1 is Double) && (param2 is Double))
            {
                return Math.Abs(((Double)param1) - ((Double)param2))
                        < tolerance;
            }

            if ((param1 is ArrayList) && (param2 is ArrayList))
            {
                // Check for equal size
                if (((ArrayList)param1).Count != ((ArrayList)param2).Count)
                    return false;
                else   // Check equality of all elements
                {
                    for (int i = 0; i < ((ArrayList)param1).Count; i++)
                        if (!equal(((ArrayList)param1)[i], ((ArrayList)param2)[i]))
                            return false;
                }
                return true;
            }
		    // test any other types here
		    return param1.Equals(param2);
	    }


    	
	    /**
	     * Helper function for printing.
	     */
	    private static void print(String str) {
		    Console.Out.Write(str);
	    }

	    /**
	     * Helper function for printing lines.
	     */
	    private static void println(String str) {
		    Console.Out.WriteLine(str);
	    }




        //protected FileStream testFile;
        //protected StreamReader fr;

        //[Test]
        //public void TestPostfixEvaluator ()
        //{
        //    testFile = new FileStream(JepTests.Properties.Resources.JEPTestExpressions, FileMode.Open);
        //    fr = new StreamReader(testFile);

        //}

        //**
        // * Reads the next line from the Reader into a  String.
        // * @throws Exception when IOException occurs, parsing fails, or when
        // *         evaluation fails
        // */
        //private String getNextLine(StreamReader reader)  {
        //    String line;
    		
        //    // cycle till a valid line is found
        //    do {
        //        line = reader.readLine(); // returns null on end of file
        //        if (line == null) return null;
        //        lineCount++;
        //    } while (line.length() == 0 || line.trim().charAt(0) == '#');
    		
        //    return line;
        //}
    }
}
