﻿using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Framework;

using SingularSys.Jep.Parser;
using SingularSys.Jep.Functions;
using SingularSys.Jep.Standard;
using SingularSys.Jep.Types;

namespace SingularSys.Jep.JepTests.Parser
{
    [TestFixture]
    public class ASTFunNodeTest
    {
        /// <summary>
        /// A test for testing basic functionality of ASTFunNode
        /// Creates a function node, adds children with integers and calculates the result
        /// </summary>
        [Test]
        public void CreateASTFunNodeTestNumbers()
        {
            ASTFunNode funNode = new ASTFunNode(1);
            funNode.JjtOpen();

            ASTConstant child1 = new ASTConstant(2);
            ASTConstant child2 = new ASTConstant(3);
            child1.Value = new JepDouble(5.0);
            child2.Value = new JepDouble(6.0);
            funNode.JjtAddChild(child1, 0);
            funNode.JjtAddChild(child2, 1);

            Add addFun = new Add();
            funNode.SetFunction("add", addFun);
            Assert.AreEqual("Function: \"add\"", funNode.ToString());
            Assert.AreEqual(2, funNode.JjtGetNumChildren());

            StandardEvaluator stdEvaluator = new StandardEvaluator();
            Object value = new Object();
            value = stdEvaluator.Eval(child1);
            Assert.AreEqual(5, ((JepDouble) value).Value);
            value = stdEvaluator.Eval(child2);
            Assert.AreEqual(6, ((JepDouble)value).Value);
            System.Console.WriteLine(funNode.JjtGetChild(0).ToString());
            try
            {
                value = stdEvaluator.Eval(funNode);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
                Console.WriteLine(e.Message);
            }
            Assert.AreEqual(11, ((JepDouble) value).Value);
        }


        /// <summary>
        /// A test for testing basic functionality of ASTFunNode
        /// Creates a function node, adds children with strings and calculates the result
        /// </summary>
        [Test]
        public void CreateASTFunNodeTestStrings()
        {
            ASTFunNode funNode = new ASTFunNode(1);
            funNode.JjtOpen();


            ASTConstant child1 = new ASTConstant(2);
            ASTConstant child2 = new ASTConstant(3);
            child1.Value = "abc";
            child2.Value = "def";
            funNode.JjtAddChild(child1, 0);
            funNode.JjtAddChild(child2, 1);

            Add addFun = new Add();
            funNode.SetFunction("add", addFun);
            Assert.AreEqual("Function: \"add\"", funNode.ToString());
            Assert.AreEqual(2, funNode.JjtGetNumChildren());

            StandardEvaluator stdEvaluator = new StandardEvaluator();
            Object value = new Object();
            value = stdEvaluator.Eval(child1);
            Assert.AreEqual("abc", value);
            value = stdEvaluator.Eval(child2);
            Assert.AreEqual("def", value);

            try
            {
                value = stdEvaluator.Eval(funNode);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
                Console.WriteLine(e.Message);
            }
            Assert.AreEqual("abcdef", value);
        }
    }
}
